#ifndef CONVERTER_H
#define CONVERTER_H

#include <iostream>
#include <fstream>
#include <string>
#include <conio.h>
#include <stdlib.h>
#include <windows.h>
#include <math.h>

#include <QDebug>
#include <QFile>
#include <QFileInfo>
#include <QSound>
#include <QString>

#include "LinkedList.h"

#define NoteOff		0x08
#define NoteOn		0x09
#define NoteAt		0x0A
#define Controller	0x0B
#define PrgmChange	0x0C
#define Aftertouch	0x0D
#define PitchBend	0x0E
#define MetaTypey	0x0F

#define EndMeta		0x2F
#define Tempo		0x51
#define Sequencer       0x7F

#define M_PI            3.14159265358979323846

using namespace std;

void ChkError(bool a, string str);
void ChkWarn(bool a, string str);

struct NOTE {
    unsigned int time;
    unsigned char freq;
};

class Converter
{
private:
    unsigned short ntable[128];
    LinkedList ndata;

    QFile* infile;

    unsigned short speed,mainspeed,tracks;
    unsigned int tracksize,totaltime;
    unsigned int deltatime;

    qint64 filestart;

public:
    Converter();
    ~Converter();

    QString open(QString filename);
    QString convert();
    int getSize() { return ndata.size(); }
    void makeWav();
    void output(QString path, bool isProg, bool playr);
    void closeFile();

    static const int samplefreq = 32000;
    static const int samples = 800;
    static const int pausetime = 340;

    int pitchshift;
    unsigned char fcuttoff;

private:
    void createBinary(unsigned int i, QFile* list);

    unsigned char read1();
    unsigned short read2();
    unsigned int read3();
    unsigned int read4();
    unsigned int readV();
    void readNibs(unsigned char &a, unsigned char &b);
    void readSkip();

    void outHex(QFile* file, unsigned char a);
    void outHex2(QFile* file, unsigned short a);

    void playNote(Link* chord, QFile* file);
    void writeNote(Link* chord, QFile* file, bool isProg);
    void writeZeros(QFile* file, int num);
    void writeAllFile(QFile* file, bool isProg, bool playr);
    void writeRaw(QFile* file, unsigned char* data, int size);
};

#endif // CONVERTER_H
